#!/usr/bin/env python3

from __future__ import print_function

import sys
import os
import argparse
import json

from nanodlna import *

def get_subtitle(file_video):

    video, extension = os.path.splitext(file_video)

    file_subtitle = "{}.srt".format(video)

    if os.path.exists(file_subtitle):
        return file_subtitle
    else:
        return None
    

def list_devices(args):

    my_devices = devices.get_devices(args.timeout)

    for i, device in enumerate(my_devices, 1):
        print("Device {}:\n{}\n\n".format(i, json.dumps(device, indent=4)))


def play(args):

    # Get video and subtitle file names

    files = { "file_video" : args.file_video }

    if args.use_subtitle:

        if not args.file_subtitle:
            args.file_subtitle = get_subtitle(args.file_video)

        if args.file_subtitle:
            files["file_subtitle"] = args.file_subtitle


    # Select device to play

    if args.device_url:
        device = devices.register_device(args.device_url)
    else:
        my_devices = devices.get_devices(args.timeout)

        if len(my_devices) > 0:
            if args.device_query:
                device = [ device for device in my_devices 
                               if args.device_query.lower() in str(device).lower() ][0]
            else:
                device = my_devices[0]
        else:
            sys.exit("No devices found.")


    # Configure streaming server

    target_ip = device["hostname"]
    serve_ip = streaming.get_serve_ip(target_ip)
    files_urls = streaming.start_server(files, serve_ip)


    # Play the video through DLNA protocol 

    dlna.play(files_urls, device)


def run():

    parser = argparse.ArgumentParser(description="A minimal UPnP/DLNA media streamer.")
    parser.add_argument("-t", "--timeout", type=float, default=5)
    subparsers = parser.add_subparsers(dest="subparser_name")

    p_list = subparsers.add_parser('list')
    p_list.set_defaults(func=list_devices)

    p_play = subparsers.add_parser('play')
    p_play.add_argument("-d", "--device", dest="device_url")
    p_play.add_argument("-q", "--query-device", dest="device_query")
    p_play.add_argument("-s", "--subtitle", dest="file_subtitle")
    p_play.add_argument("-n", "--no-subtitle", dest="use_subtitle", action="store_false")
    p_play.add_argument("file_video")
    p_play.set_defaults(func=play)

    args = parser.parse_args()
    args.func(args)


if __name__ == "__main__":

    run()




#!/usr/bin/env python3
# encoding: UTF-8

import os
import socket
import threading

from twisted.web.server import Site
from twisted.web.static import File
from twisted.web.resource import Resource
from twisted.internet import reactor
from twisted.python import log


def set_files(files, serve_ip, serve_port):

    files_index = { file_key : ( os.path.basename(file_path), 
                                 os.path.abspath(file_path),
                                 os.path.dirname(os.path.abspath(file_path)))
                             for file_key, file_path in files.items() }

    files_serve = { file_name : file_path 
                        for file_name, file_path, file_dir in files_index.values() }

    files_urls = { file_key : "http://{}:{}/{}/{}".format(serve_ip, serve_port, file_key, file_name) 
                       for file_key, (file_name, file_path, file_dir) in files_index.items() }

    return files_index, files_serve, files_urls


def start_server(files, serve_ip, serve_port=9000):

    import sys
    #log.startLogging(sys.stdout)

    files_index, files_serve, files_urls = set_files(files, serve_ip, serve_port)

    root = Resource()
    for file_key, (file_name, file_path, file_dir) in files_index.items():
        root.putChild(file_key.encode("utf-8"), Resource())
        root.children[file_key.encode("utf-8")].putChild(file_name.encode("utf-8"), File(file_path))

    reactor.listenTCP(serve_port, Site(root))
    threading.Thread(target=reactor.run, kwargs={"installSignalHandlers":False}).start()

    return files_urls


def get_serve_ip(target_ip, target_port=80):
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.connect((target_ip, target_port))
    serve_ip = s.getsockname()[0]
    s.close()
    return serve_ip


if __name__ == "__main__":

    import sys

    files = {"file_{}".format(i) : file_path  for i, file_path in enumerate(sys.argv[1:], 1)} 
    print(files)

    files_urls = start_server(files, "localhost")
    print(files_urls)




#!/usr/bin/env python3
# encoding: UTF-8

import sys

if sys.version_info.major == 3:
    import urllib.request as urllibreq
else:
    import urllib2 as urllibreq

import os
from xml.sax.saxutils import escape as xmlescape
import pkgutil


def send_dlna_action(device, data, action):

    action_data = pkgutil.get_data("nanodlna", "templates/action-{}.xml".format(action)).decode("UTF-8")
    action_data = action_data.format(**data).encode("UTF-8")

    headers = { 
      "Content-Type"   : "text/xml; charset=\"utf-8\"",
      "Content-Length" : "{}".format(len(action_data)),
      "Connection"     : "close",
      "SOAPACTION"     : "\"{}#{}\"".format(device["st"], action)
    } 

    request = urllibreq.Request(device["action_url"], action_data, headers)
    urllibreq.urlopen(request)


def play(files_urls, device):

    video_data = {
        "uri_video"  : files_urls["file_video"],
        "type_video" : os.path.splitext(files_urls["file_video"])[1][1:],
    }

    if "file_subtitle" in files_urls and files_urls["file_subtitle"]:

        video_data.update({
            "uri_sub"    : files_urls["file_subtitle"],
            "type_sub"   : os.path.splitext(files_urls["file_subtitle"])[1][1:]
        })

        metadata = pkgutil.get_data("nanodlna", "templates/metadata-video_subtitle.xml").decode("UTF-8")
        video_data["metadata"] = xmlescape(metadata.format(**video_data))

    else:
        video_data["metadata"] = ""

    send_dlna_action(device, video_data, "SetAVTransportURI")
    send_dlna_action(device, video_data, "Play")

